# Generated by Django 3.2 on 2021-04-10 15:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crudlfap_sites', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='site',
            name='settings',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
